function channelRegistration(start_DataDir, Indx, Channels, ChannelLabel, ...
                             NNeighborPoints)
% Perform registration between channels 2 through the last channel with
% respect to fixed channel 1.  Files GeomTransform_*.mat will be produced in
% the same directory as the fiducial data, which are assumed to be in the
% 'fiducial' subdirectory of the directory containing the files to be tracked.

% ##### IMPORTANT #####
% Channel 1 is fixed and channel 2 is transformed onto channel 1

% The data are organized as follows.  Each channel is one quadrant of an image
% split into 4 quadrants, corresponding to a given wavelength/color (see
% ChannelLabel).  Some frames are excluded from consideration for registration
% (see definitions below).  The odd numbered frames are RegDataChannel1, while
% the even frames are RegDataChannel2 over which registration takes place.

% INPUTS:
%    start_DataDir     start location to look for data files
%    Indx              x and y quadrant indices for DIPimage format---see
%                      FiducialChannel definitions below
%    Channels          channel numbers that are being used
%    ChannelLabel      cell array of strings identifying each channel
%    NNeighborPoints   must be at least 6 but 6 sometimes gives errors

[FileNameRegData, FilePathRegData] = ...
        uigetfile(fullfile(start_DataDir, '*.mat'), ...
        'Select the channel registration data');
load([fullfile(FilePathRegData, FileNameRegData(1:end-4)), '.mat'], ...
        'sequence');

% [BL, TL, TR, BR] = SiMPullUtils.quadView_SplitDIPimage(sequence);
% FiducialChannel{1} = TR(:, :, 50:99);   % Top Right
% FiducialChannel{2} = TL(:, :, 50:99);   % Top Left
% FiducialChannel{3} = BL(:, :, 50:99);   % Bottom Left
% FiducialChannel{4} = BR(:, :, 50:99);   % Bottom Right
sequence = dip_image(sequence);
FiducialChannel{1} = sequence(Indx{1}{1}, Indx{1}{2},:);   % Top Right
FiducialChannel{2} = sequence(Indx{2}{1}, Indx{2}{2}, :);   % Top Left
FiducialChannel{3} = sequence(Indx{3}{1}, Indx{3}{2}, :);   % Bottom Left
% %FiducialChannel{4} = sequence(Indx{4}{1}, Indx{4}{2}, 50:99);   % Bottom Right


% To use smi_core.ChannelRegistration easily, save the data in temporary files
% here, then read them back in when doing channel registration in the next
% loop.
for i = 1 : max(Channels)
   if ~ismember(i, Channels)
      continue;
   end

   RegDataChannel_1 = single(FiducialChannel{i});
   RegDataChannel_1 = mean(RegDataChannel_1(:, :, 1:2:end), 3);
   RegDataChannel_2 = single(FiducialChannel{i});
   RegDataChannel_2 = mean(RegDataChannel_2(:, :, 2:2:end), 3);
   sequence = RegDataChannel_1;
   save(fullfile(FilePathRegData, sprintf('RegDataChannel%d_1', i)), ...
        'sequence');
   sequence = RegDataChannel_2;
   save(fullfile(FilePathRegData, sprintf('RegDataChannel%d_2', i)), ...
        'sequence');
end

for i = 2 : max(Channels)

   if ~ismember(i, Channels)
      continue;
   end

   %ChannelRegData=(single(FiducialChannel{1}));
   %RegDataChannel1 = mean(ChannelRegData(:, :, 1:2:end), 3);

   %ChannelRegData=(single(FiducialChannel{i}));
   %RegDataChannel2 = mean(ChannelRegData(:, :, 2:2:end), 3);

   FiducialFileNames = {sprintf('RegDataChannel%d_1.mat', 1), ...
                        sprintf('RegDataChannel%d_2.mat', i)};
   load(fullfile(FilePathRegData, FiducialFileNames{1}));
   RegDataChannel1 = sequence;
   load(fullfile(FilePathRegData, FiducialFileNames{2}));
   RegDataChannel2 = sequence;

   SMFChannel12 = smi_core.SingleMoleculeFitting();
      % Minimum number of photons from emitter (Default=200)
   SMFChannel12.BoxFinding.MinPhotons    = 200;
      % Initial or fixed Sigma of 2D Gaussian PSF Model (Pixels)(Default=1)
   SMFChannel12.Fitting.PSFSigma         = 1;
      % Maximum PSF Sigma from fit (Pixels)(Default=2)
   SMFChannel12.Thresholding.MaxPSFSigma = inf;
      % Minimum accepted photons from fit (Default=100)
   SMFChannel12.Thresholding.MinPhotons  = 100;
      % Minimum accepted p-value from fit (Default=.01)
%    SMFChannel12.Thresholding.MinPValue   = 0.01;
    SMFChannel12.Thresholding.MinPValue   = 0;

   % RegDataChannel1 is the reference fiducial and RegDataChannel2 is the
   % fidicual to be registered with respect to RegDataChannel1.
   Verboze = 1;
   ChannelReg = smi_core.ChannelRegistration( ...
                   FilePathRegData, FiducialFileNames, SMFChannel12, Verboze);
   ChannelReg.TransformationType = 'lwm';
   ChannelReg.TransformationBasis = 'coordinates';
   %ChannelReg.NNeighborPoints = 12;
   ChannelReg.NNeighborPoints = NNeighborPoints;
   ChannelReg.findTransform();
   GeomTransform = ChannelReg.RegistrationTransform{2};
   % Apply the geometric transform to the raw data (we will NOT analyze the
   % transformed data, this is only used for display purposes in the final
   % movie).
   [CoordRefObject] = imref2d([size(RegDataChannel2, 1), ...
       size(RegDataChannel2, 2)]);

   % Create an overlay of the registered fiducials to check if the transform
   % worked.
   OverlayImage = zeros([size(RegDataChannel1), 3]);
   OverlayRegImage = zeros([size(RegDataChannel1), 3]);
   RegDataChannel2Registered = imwarp(RegDataChannel2, GeomTransform, ...
       'OutputView', CoordRefObject);
   OverlayImage(:, :, 1) = RegDataChannel1 ./ max(RegDataChannel1(:));
   OverlayImage(:, :, 2) = RegDataChannel2 ./ max(RegDataChannel2(:));
   OverlayRegImage(:, :, 1) = RegDataChannel1 ./ max(RegDataChannel1(:));
   OverlayRegImage(:, :, 2) = RegDataChannel2Registered ...
       ./ max(RegDataChannel2Registered(:));
   figure('Name', 'Raw Overlay');
   %imshow(OverlayImage)
   image(OverlayImage)
   figure('Name', 'Registered Overlay');
   %imshow(OverlayRegImage)
   image(OverlayRegImage)

   ChannelRegInfo.SMFChannel12 = SMFChannel12;
   ChannelRegInfo.NNeighborPoints = NNeighborPoints;
   ChannelRegInfo.GeomTransform = GeomTransform;
   ParamStruct.ChannelRegInfo = ChannelRegInfo; 

   TransformFilename = sprintf('GeomTransform_%s', ChannelLabel{i});
   fprintf('Saving %s ...\n', TransformFilename);
   save(fullfile(FilePathRegData, TransformFilename), 'ParamStruct');

end % i

fprintf('Done channel registration.\n');

end
