function [gain, offset] = findGainOffset(start_DataDir)
%findGainOffset finds the camera gain and offset for a stack of input images
% with respect to a stack of background images.  Use DIPimage's cal_readnoise.
%
% INPUTS:
%    start_DataDir   start location to look for data files
%
% OUTPUTS:
%    gain     camera gain   (ADU / e-)
%    offset   camera offset (ADU)
%
% From the cal_readnoise documentation:
%
%    out = cal_readnoise(in, bg)
%
% where
%
%    in: input image (a stack of images -at least 10- containing repeated
%        measurements of the same object); best if all possible intensities
%        are present in the image (create a intensity gradient, e.g.
%        out-of-focus edge)
%    bg: background image (better a series of background images)
%
% and
%
%     out(1) n(e-) variance in the background
%     out(2) the 1/slope of the fit in [photons per ADU]:
%            conversion ADU = detected #photons * slope
%     out(3) n(e-) readnoise fit; sqrt(of y-axis intersection) / slope
%     out(4) mean background

   [FileNameIn, FilePathIn] = uigetfile(fullfile(start_DataDir, '*.mat'), ...
                                        'Select the gain image stack');
   [FileNameBg, FilePathBg] = uigetfile(fullfile(start_DataDir, '*.mat'), ...
                                        'Select the background image stack');
   in = load(fullfile(FilePathIn, FileNameIn), 'sequence');
   bg = load(fullfile(FilePathBg, FileNameBg), 'sequence');

   close all
   out = cal_readnoise(in.sequence, bg.sequence);

   f1 = figure(1);
   f2 = figure(2);
   saveas(f1, fullfile(FilePathIn, 'GainOffset1'), 'png');
   saveas(f2, fullfile(FilePathIn, 'GainOffset2'), 'png');
   gain   = 1 / out(2);
   offset = out(4);
   fprintf('gain = %f ADU/e-, offset = %f ADU\n', gain, offset);
   fprintf('Done finding gain and offset.\n');

end
