function statsCorrectedBG(start_DataDir, Channels)
% Correction: NGFP = (NLOC - NBG)*SR where
% NGFP is the corrected number of surface GFP receptors,
% NLOC is the total number of emitters localized,
% NBG  is the expected number of background emitters in the area imaged, and
% SR (surface ratio) is the fraction of receptors located at the cell surface.
%
% INPUTS:
%    start_DataDir   start location to look for data files
%    Channels        channel numbers that are being used

% Script to do statistical analysis of SiMPull results and plot them.

% v3_03: this version have error bars based on a binomial distribution. The
% reason is that each receptor sampled resembles a Bernoulli trial.

% v3_02: this version corrects the total number of EGFR-GFPs (488)
% substracting background spots and accounting only receptors at cell
% surface. Additionally, error bars are calculated for phosphorylation
% percentages considering molecules are randomly distributed (Spatial
% Poisson distribution).

% v3: this version adds all the countings from all fields of views and then
% gets the % phosphorylation, instead of getting percentages for each field
% of view and then get the average of those. It also substracts the average
% number of background spots in the GFP channel for each field of view.

% Define date of experiment and tag of analyzed data
% dateExp= '170612'; % Date of the experiment. E.g. 160728
[FileName, FilePath] = uigetfile(fullfile(start_DataDir, 'results_*.mat'), ...
                                 'Choose results_* file');
dateExp = regexprep(FileName, 'results_(.*)\.mat', '$1');
SaveDir = fullfile(FilePath, 'Corrected');
if ~isfolder(SaveDir)
   mkdir(SaveDir);
end

% ----------
% conditionsCorrectedBG should exist in the directory containing the results.mat
% file chosen above.  conditionsCorrectedBG contains the conditions/labels/etc.
% needed for the analysis below.  See this file for further details.
conditions{1} = [];  % initialized to avoid syntax error
run(fullfile(FilePath, 'conditionsCorrectedBG.m'));
% ----------

% Load analyzed SiMPull results
% loads cell array called 'valuecell' from a previous results.mat file.
% load(['results_' dateExp '.mat']);
load(fullfile(FilePath, FileName));
% If you need to concatenate cell arrays from different analysis
% use [valuecell; valuecell]

% Get background counts on 488 channel
% Find background data. Set in blank everything that doesn't have the keyword
% specified, gives back position where the keyword is inside cells (not cell
% position)
findCondBG= strfind(valuecell(:,1), nameBgFile);
% Index of cells containing background data
indexCondBG= find(~cellfun('isempty',findCondBG));
meanBG= mean(cell2mat(valuecell(indexCondBG,2)));

% Process and plot the different sets of experiments/conditions (e.g., kinetics
% of EGFR phosphorylation in WT vs mutant cells).
for iiExp=1:length(tagExp)

   xCondPlot{iiExp}= 1:length(conditions{iiExp});

   % Initialize cell array for index for diferent conditions
   indexCond{iiExp}= cell(conditions{iiExp});

   for ii=1:size(conditions{iiExp},2)
      % Find condition. Set in blank everything that doesn't have the keyword
      % specified, gives back position were the keyword is inside cells (not
      % cell position)
      findCond= strfind(valuecell(:,1),conditions{iiExp}{ii});
      % Index of cells of current condition
      indexCond{iiExp}{ii}= find(~cellfun('isempty',findCond));
      simpull{iiExp}.condition{ii}= conditions{iiExp}{ii};

      % Total number of 488, 561 and 642 spots
      % IMPORTANT: total number of EGFR-GFPs (488) are corrected substracting
      % background spots and accounting only for receptors at cell surface
      simpull{iiExp}.tot488{ii}= cell2mat(valuecell(indexCond{iiExp}{ii},2));
      simStats{iiExp}.tot488Sum(ii)= sum(simpull{iiExp}.tot488{ii});
      % Surface receptors (corrected) = 
      %    (N of fluorophores - <Background>)*Surface ratio
      % where fluorophores = fits or detections,
      %    <Background> = meanBG * # files over which N of fluoro. was computed
      % is the number of background spots, and Surface ratio is the fraction of
      % receptors located at the cell surface.
      simStats{iiExp}.tot488SumCorrected(ii)= ...
         round((simStats{iiExp}.tot488Sum(ii)-meanBG*length(simpull{iiExp}.tot488{ii}))*surface_ratio);

      simpull{iiExp}.tot561{ii}= cell2mat(valuecell(indexCond{iiExp}{ii},3));
      simStats{iiExp}.tot561Sum(ii)= sum(simpull{iiExp}.tot561{ii});

      simpull{iiExp}.tot642{ii}= cell2mat(valuecell(indexCond{iiExp}{ii},4));
      simStats{iiExp}.tot642Sum(ii)= sum(simpull{iiExp}.tot642{ii});

      % Number of overlapping spots
      simpull{iiExp}.ov488w561{ii}= cell2mat(valuecell(indexCond{iiExp}{ii},5));
      simStats{iiExp}.ov488w561Sum(ii)= sum(simpull{iiExp}.ov488w561{ii});

      simpull{iiExp}.ov488w642{ii}= cell2mat(valuecell(indexCond{iiExp}{ii},6));
      simStats{iiExp}.ov488w642Sum(ii)= sum(simpull{iiExp}.ov488w642{ii});

      simpull{iiExp}.ov488w561w642{ii}= ...
         cell2mat(valuecell(indexCond{iiExp}{ii},8));
      simStats{iiExp}.ov488w561w642Sum(ii)= ...
         sum(simpull{iiExp}.ov488w561w642{ii});

      % Percentages of overlapping spots
      % Error bars are calculated based on a binomial distribution
      [simStats{iiExp}.perc488w561(ii), simStats{iiExp}.perc488w561SEM(ii)]= ...
          SiMPullUtils.errorBinomial(simStats{iiExp}.ov488w561Sum(ii),       ...
                                     simStats{iiExp}.tot488SumCorrected(ii));

      [simStats{iiExp}.perc488w642(ii), simStats{iiExp}.perc488w642SEM(ii)]= ...
          SiMPullUtils.errorBinomial(simStats{iiExp}.ov488w642Sum(ii),       ...
                                     simStats{iiExp}.tot488SumCorrected(ii));

      % Percentage of triple overlap
      [simStats{iiExp}.perc488w561642(ii),     ...
       simStats{iiExp}.perc488w561642SEM(ii)]= ...
          SiMPullUtils.errorBinomial(simStats{iiExp}.ov488w561w642Sum(ii), ...
                                     simStats{iiExp}.tot488SumCorrected(ii));

      % Percentage of 488-561 overlapping with 642
      [simStats{iiExp}.perc488561w642(ii),     ...
       simStats{iiExp}.perc488561w642SEM(ii)]= ...
          SiMPullUtils.errorBinomial(simStats{iiExp}.ov488w561w642Sum(ii), ...
                                     simStats{iiExp}.ov488w561Sum(ii));

      % Percentage of 488-642 overlapping with 561
      [simStats{iiExp}.perc488642w561(ii),     ...
       simStats{iiExp}.perc488642w561SEM(ii)]= ...
          SiMPullUtils.errorBinomial(simStats{iiExp}.ov488w561w642Sum(ii), ...
                                     simStats{iiExp}.ov488w642Sum(ii));

   end % ii

   % Define column identifiers and save results in excel files (without
   % Standard Error in first sheet and with SEM in the second one)
   colIdent= {'Condition', '488-fits','488-fits-corrected','561-fits','642-fits','488w561','488w642','488w561-642'...
       '% 488 w/561','SEM','% 488 w/642','SEM','% 488 w/561-642','SEM','% 488-561 w/642','SEM','% 488-642 w/561','SEM'};

%   statsCell= cell(length(simpull{iiExp}.condition)+1,numel(colIdent));
%   statsCell(1,:)= colIdent;
%   statsCell(2:end,1)= simpull{iiExp}.condition';
%   statsCell(2:end,2)= num2cell(simStats{iiExp}.tot488Sum)';
%   statsCell(2:end,3)= num2cell(simStats{iiExp}.tot488SumCorrected)';
%   statsCell(2:end,4)= num2cell(simStats{iiExp}.tot561Sum)';
%   statsCell(2:end,5)= num2cell(simStats{iiExp}.tot642Sum)';
%   statsCell(2:end,6)= num2cell(simStats{iiExp}.ov488w561Sum)';
%   statsCell(2:end,7)= num2cell(simStats{iiExp}.ov488w642Sum)';
%   statsCell(2:end,8)= num2cell(simStats{iiExp}.ov488w561w642Sum)';
%
%   statsCell(2:end,9)= num2cell(simStats{iiExp}.perc488w561)';
%%  statsCell(2:end,10)= num2cell(simStats{iiExp}.perc488w561SEM)';
%   statsCell(2:end,11)= num2cell(simStats{iiExp}.perc488w642)';
%%  statsCell(2:end,12)= num2cell(simStats{iiExp}.perc488w642SEM)';
%   statsCell(2:end,13)= num2cell(simStats{iiExp}.perc488w561642)';
%%  statsCell(2:end,14)= num2cell(simStats{iiExp}.perc488w561642SEM)';
%   statsCell(2:end,15)= num2cell(simStats{iiExp}.perc488561w642)';
%%  statsCell(2:end,16)= num2cell(simStats{iiExp}.perc488561w642SEM)';
%   statsCell(2:end,17)= num2cell(simStats{iiExp}.perc488642w561)';
%%  statsCell(2:end,18)= num2cell(simStats{iiExp}.perc488642w561SEM)';
%
%   % data saved in excel file
%   try
%   xlswrite(fullfile(SaveDir,                                             ...
%                     ['resultsStats_' dateExp '_' tagExp{iiExp} '.xls']), ...
%            statsCell, 'Mean')
%   catch Me
%   end

   statsCell= cell(length(simpull{iiExp}.condition)+1,numel(colIdent));
   statsCell(1,:)= colIdent;
   statsCell(2:end,1)= simpull{iiExp}.condition';
   statsCell(2:end,2)= num2cell(simStats{iiExp}.tot488Sum)';
   statsCell(2:end,3)= num2cell(simStats{iiExp}.tot488SumCorrected)';
   statsCell(2:end,4)= num2cell(simStats{iiExp}.tot561Sum)';
   statsCell(2:end,5)= num2cell(simStats{iiExp}.tot642Sum)';
   statsCell(2:end,6)= num2cell(simStats{iiExp}.ov488w561Sum)';
   statsCell(2:end,7)= num2cell(simStats{iiExp}.ov488w642Sum)';
   statsCell(2:end,8)= num2cell(simStats{iiExp}.ov488w561w642Sum)';
   statsCell(2:end,9)= num2cell(simStats{iiExp}.perc488w561)';
   statsCell(2:end,10)= num2cell(simStats{iiExp}.perc488w561SEM)';
   statsCell(2:end,11)= num2cell(simStats{iiExp}.perc488w642)';
   statsCell(2:end,12)= num2cell(simStats{iiExp}.perc488w642SEM)';
   statsCell(2:end,13)= num2cell(simStats{iiExp}.perc488w561642)';
   statsCell(2:end,14)= num2cell(simStats{iiExp}.perc488w561642SEM)';
   statsCell(2:end,15)= num2cell(simStats{iiExp}.perc488561w642)';
   statsCell(2:end,16)= num2cell(simStats{iiExp}.perc488561w642SEM)';
   statsCell(2:end,17)= num2cell(simStats{iiExp}.perc488642w561)';
   statsCell(2:end,18)= num2cell(simStats{iiExp}.perc488642w561SEM)';

   % data saved in excel file
   try
   xlswrite(fullfile(SaveDir,                                             ...
                     ['resultsStats_' dateExp '_' tagExp{iiExp} '.xls']), ...
            statsCell, 'Mean and SEM')
   catch ME
   end

   switch graphOption
      case 1
         % Create figure
         %close all
         % To specify white background instead of gray use [1 1 1]
         h1=figure('Color',[1 1 1]);

         % Plot with errorbars
         if ismember(2, Channels) && ismember(3, Channels)
            h = SiMPullUtils.barwitherr([simStats{iiExp}.perc488w561SEM;   ...
                                         simStats{iiExp}.perc488w642SEM]', ...
                                        [simStats{iiExp}.perc488w561;      ...
                                         simStats{iiExp}.perc488w642]');
         elseif ismember(2, Channels)   % 3 is NOT in Channels
            h = SiMPullUtils.barwitherr([simStats{iiExp}.perc488w561SEM],  ...
                                        [simStats{iiExp}.perc488w561]);
         elseif ismember(3, Channels)   % 2 is NOT in Channels
            h = SiMPullUtils.barwitherr([simStats{iiExp}.perc488w642SEM],  ...
                                        [simStats{iiExp}.perc488w642]);
         else   % 2 and 3 are NOT in Channels
            return;
         end
         title(titleGraph{iiExp},'FontSize',26)
         ylabel(yLabelGraph{iiExp},'FontSize',24)

         set(gca,'XTickLabel',nameGroups{iiExp})
         legend(nameColumns{iiExp})
         % set(h(1),'FaceColor',[0.7 0.7 0.7]);
         set(gca,'fontsize',20)
         % set(h1,'WindowStyle', 'docked')
         % find handle for error bars and set LineWidth
         set(findall(h1,'LDataSource',''),'LineWidth',2)
         % set(gca,'DataAspectRatio',[1 25 1])
         % h1=findobj('-property','EdgeColor')
         % set(h1(1),'EdgeColor','none')

         % Save figure
         %export_fig(fullfile(SaveDir,                                      ...
         %                    ['graph_' dateExp '_' tagExp{iiExp} '.png']), ...
         %           '-m2')
         saveas(gcf, ...
                fullfile(SaveDir, ['graph_' dateExp '_' tagExp{iiExp} '.png']))
         savefig(fullfile(SaveDir, ['graph_' dateExp '_' tagExp{iiExp} '.fig']))

      case 2
         % Create figure and give white background
         % To specify white background instead of gray use [1 1 1]
         h1=figure('Color',[1 1 1]);

         errorbar(xValPlot{iiExp}, simStats{iiExp}.perc488w561,         ...
                                   simStats{iiExp}.perc488w561SEM,      ...
                  [colorList{iiExp} makerList{iiExp}], 'MarkerSize', 5, ...
                  'MarkerFaceColor', colorList{iiExp}, 'LineWidth', 2,  ...
                  'DisplayName', conditionsName{iiExp})
         hold on
         xlim([0 maxXLim])
         ylim([0 maxYLim])
         title(titleGraph,'FontSize',36)
         xlabel(xLabelGraph,'FontSize',36)
         ylabel(yLabelGraph,'FontSize',36)
         set(gca,'fontsize',30)
         % set(h1,'WindowStyle', 'docked')
         set(gca,'DataAspectRatio',[3 1 1])

   end % switch graphOption

   % Copy tables for each tagExp{iiExp} in single temporary
   % variable to be able to save it into an excel file
   simpullTemp= simpull{iiExp};
   simStatsTemp= simStats{iiExp};

   save(fullfile(SaveDir, ['table_' dateExp '_' tagExp{iiExp} '.mat']), ...
        'simpullTemp', 'simStatsTemp')

end % iiExp

switch graphOption
   case 1

   case 2
      % Save figure
      legend('show')
      %export_fig(fullfile(SaveDir, ['graph_' dateExp '_' nameGraph '.png']), ...
      %           '-m2')
      saveas(gcf, fullfile(SaveDir, ['graph_' dateExp '_' nameGraph '.png']))
      savefig(fullfile(SaveDir, ['graph_' dateExp '_' nameGraph '.fig']))

end

fprintf('Done statstics for corrected BG.\n');

end
